#include <avr/interrupt.h>
#include <avr/io.h>

#include <stdio.h>
#include <util/atomic.h>

#include "../include/uCTest/uCTest.h"
#include "System.h"

#define SOR ( *( ( volatile char * ) 0x20 ) )
#define SEXITR ( *( ( volatile char * ) 0x21 ) )

const stack_t *uctest_stackStart = ( ( stack_t * ) RAMEND );

static int uart_putchar( char c, FILE *stream );
static FILE mystdout = FDEV_SETUP_STREAM( uart_putchar, NULL, _FDEV_SETUP_WRITE );
static int uart_putchar( char c, FILE *stream ) {
    ( ( void ) stream );
    SOR = c;
    return 0;
}

void __attribute__( ( naked, section( ".init8" ) ) ) uctest_system_init_hardware( void ) {
    stdout = &mystdout;

#ifdef CONFIC_COUNT_CYCLES
    ATOMIC_BLOCK( ATOMIC_FORCEON ) {
        TCCR1B = ( 1 << CS10 );
        TIMSK1 = ( 1 << TOIE1 );
    }
#endif
}

void __attribute__( ( naked, section( ".init6" ) ) ) uctest_endTestsList( void ) {
    uctest_stack_pushPtr( NULL );
}

#ifdef CONFIC_COUNT_CYCLES
static volatile uint16_t ovf;
static volatile uint16_t clk_start;

void uctest_clkMeasureStart( void ) {
    ATOMIC_BLOCK( ATOMIC_RESTORESTATE ) {
        ovf = 0;
        clk_start = TCNT1;
    }
}

void uctest_clkMeasureStop( unsigned long *clk ) {
    uint16_t clk_end;
    uint16_t ovf_end;
    ATOMIC_BLOCK( ATOMIC_RESTORESTATE ) {
        clk_end = TCNT1;
        ovf_end = ovf;
    }
    *clk += ( ( ovf_end * ( ( ( unsigned long ) UINT16_MAX ) + 1 ) ) + ( clk_end - clk_start ) );
}

ISR( TIMER1_OVF_vect ) {
    ovf++;
}
#endif

void exit( int status ) {
    __builtin_avr_cli();
    SEXITR = status;
    while( 1 )
        ;
}
