#include <stdio.h>

#include "../include/uCTest/uCTest.h"

#define XSTR( X ) UCTEST_XSTR( X )

const __flash char uctest_eqFailMsg[] =
    "Expected equality of these values :\n %s\n   Which is : %d\n %s\n   Which is : %d\n";

const __flash char uctest_nearFailMsg[] =
    "The difference between %s and %s is %d, which exceeds %s, where\n  %s "
    "evaluates to %d\n  %s evaluates to %d, and\n  %s evaluates to %d.\n";

const __flash char uctest_simpleFailMsg[] = "Expected: %s, actual: %d vs %d\n";

const __flash char uctest_deathFailMsg[] =
    "Death test: %s\n    Result: failed to die.\n[  DEATH   ]\n";

void uctest_printRuntimeStats( int suites, int tests ) {
    printf_P( XSTR( "\e[32m[==========]\e[0m Running %d tests from %d test suites.\n" ), tests,
              suites );
}

void uctest_printRuntimeStatsEnd( int suites, int tests, unsigned long cycles ) {
    printf_P(
        XSTR( "\e[32m[==========]\e[0m %d tests from %d test suites ran. (%lu cycles total)\n" ),
        suites, tests, cycles );
}

void uctest_printStartup( int suites, int tests ) {
    printf_P( XSTR( "Running main\n" ) );
    uctest_printRuntimeStats( suites, tests );
}

void uctest_printGlobalSetup( void ) {
    printf_P( XSTR( "\e[32m[----------]\e[0m Global test environment set-up.\n" ) );
}

void uctest_printGlobalTeardown( void ) {
    printf_P( XSTR( "\e[32m[----------]\e[0m Global test environment tear-down.\n" ) );
}

void uctest_printSuiteStats( const __flash char *suite, int tests ) {
    printf_P( XSTR( "\e[32m[----------]\e[0m %d tests from %S" ), tests, suite );
}

void uctest_printSuiteStatsBegin( const __flash char *suite, int tests ) {
    uctest_printSuiteStats( suite, tests );
    printf( "\n" );
}

void uctest_printSuiteStatsEnd( const __flash char *suite, int tests, unsigned long cycles ) {
    uctest_printSuiteStats( suite, tests );
    printf_P( XSTR( " (%lu cycles)\n" ), cycles );
    printf( "\n" );
}

void uctest_printTest( const __flash char *mode, const __flash struct uctest_test *test ) {
    printf_P( XSTR( "\e[32m[ %S ]\e[0m %S.%S" ), mode, test->suite_name, test->test_name );
}

void uctest_printTestBegin( const __flash struct uctest_test *test ) {
    uctest_printTest( XSTR( "RUN     " ), test );
    printf( "\n" );
}

void uctest_printTestSuccess( const __flash struct uctest_test *test, unsigned long cycles ) {
    uctest_printTest( XSTR( "      OK" ), test );
    printf_P( XSTR( " (%lu cycles)\n" ), cycles );
}

void uctest_printAllTestsPassed( int tests ) {
    printf_P( XSTR( "\e[32m[  PASSED  ]\e[0m %d tests.\n" ), tests );
}

void uctest_printFailedTestsAtEnd( int tests ) {
    printf_P( XSTR( "\n%d FAILED TESTS\n" ), tests );
}

void uctest_printTestFailure( const __flash struct uctest_test *test, unsigned long cycles ) {
    printf_P( XSTR( "\e[31m[  FAILED  ]\e[0m %S.%S" ), test->suite_name, test->test_name );
    printf_P( XSTR( " (%lu cycles)\n" ), cycles );
}

void uctest_printFaildTestNum( int tests ) {
    printf_P( XSTR( "\e[31m[  FAILED  ]\e[0m %d tests, listed below:\n" ), tests );
}

void uctest_printFailureLocation( const char *file, int line ) {
    printf_P( XSTR( "%s:%d: Failure\n" ), file, line );
}

void uctest_printUserMessage( const __flash char *fmt, va_list args ) {
    vfprintf_P( stdout, fmt, args );
}
